@@ -1,5 +1,5 @@
 //
-// Copyright 2012-2013 Ettus Research LLC
+// Copyright 2012-2014 Ettus Research LLC
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 #include <uhd/utils/cast.hpp>
 #include <uhd/exception.hpp>
 #include <uhd/utils/static.hpp>
-#include <uhd/utils/images.hpp>
+#include <uhd/utils/paths.hpp>
 #include <uhd/utils/safe_call.hpp>
 #include <boost/format.hpp>
 #include <boost/assign/list_of.hpp>
@@ -82,7 +82,7 @@ static device_addrs_t b100_find(const device_addr_t &hint)
             b100_fw_image = find_image_path(hint.get("fw", B100_FW_FILE_NAME));
         }
         catch(...){
-            UHD_MSG(warning) << boost::format("Could not locate B100 firmware. %s\n") % print_images_error();
+            UHD_MSG(warning) << boost::format("Could not locate B100 firmware. %s\n") % print_utility_error("uhd_images_downloader.py");
             return b100_addrs;
         }
         UHD_LOG << "the firmware image: " << b100_fw_image << std::endl;
@@ -137,7 +137,7 @@ static device::sptr b100_make(const device_addr_t &device_addr){
 }
 
 UHD_STATIC_BLOCK(register_b100_device){
-    device::register_device(&b100_find, &b100_make);
+    device::register_device(&b100_find, &b100_make, device::USRP);
 }
 
 /***********************************************************************
@@ -148,7 +148,9 @@ b100_impl::b100_impl(const device_addr_t &device_addr){
     b100_impl_constructor_begin:
     initialization_count++;
 
+    _type = device::USRP;
     _tree = property_tree::make();
+    _ignore_cal_file = device_addr.has_key("ignore-cal-file");
 
     //extract the FPGA path for the B100
     std::string b100_fpga_image = find_image_path(
@@ -530,10 +532,10 @@ void b100_impl::check_fw_compat(void){
     );
     if (fw_compat_num != B100_FW_COMPAT_NUM){
         throw uhd::runtime_error(str(boost::format(
-            "Expected firmware compatibility number 0x%x, but got 0x%x:\n"
+            "Expected firmware compatibility number %d, but got %d:\n"
             "The firmware build is not compatible with the host code build.\n"
             "%s"
-        ) % B100_FW_COMPAT_NUM % fw_compat_num % print_images_error()));
+        ) % int(B100_FW_COMPAT_NUM) % fw_compat_num % print_utility_error("uhd_images_downloader.py")));
     }
     _tree->create<std::string>("/mboards/0/fw_version").set(str(boost::format("%u.0") % fw_compat_num));
 }
@@ -550,7 +552,7 @@ void b100_impl::check_fpga_compat(void){
             "Expected FPGA compatibility number %d, but got %d:\n"
             "The FPGA build is not compatible with the host code build."
             "%s"
-        ) % int(B100_FPGA_COMPAT_NUM) % fpga_major % print_images_error()));
+        ) % int(B100_FPGA_COMPAT_NUM) % fpga_major % print_utility_error("uhd_images_downloader.py")));
     }
     _tree->create<std::string>("/mboards/0/fpga_version").set(str(boost::format("%u.%u") % fpga_major % fpga_minor));
 }
@@ -608,9 +610,13 @@ sensor_value_t b100_impl::get_ref_locked(void){
 }
 
 void b100_impl::set_rx_fe_corrections(const double lo_freq){
-    apply_rx_fe_corrections(this->get_tree()->subtree("/mboards/0"), "A", lo_freq);
+    if(not _ignore_cal_file){
+        apply_rx_fe_corrections(this->get_tree()->subtree("/mboards/0"), "A", lo_freq);
+    }
 }
 
 void b100_impl::set_tx_fe_corrections(const double lo_freq){
-    apply_tx_fe_corrections(this->get_tree()->subtree("/mboards/0"), "A", lo_freq);
+    if(not _ignore_cal_file){
+        apply_tx_fe_corrections(this->get_tree()->subtree("/mboards/0"), "A", lo_freq);
+    }
 }
