@@ -1,5 +1,5 @@
 //
-// Copyright 2013-2014 Ettus Research LLC
+// Copyright 2013-2015 Ettus Research LLC
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -49,13 +49,14 @@
 #include <uhd/transport/nirio/niusrprio_session.h>
 #include <uhd/transport/vrt_if_packet.hpp>
 #include "recv_packet_demuxer_3000.hpp"
+#include "x300_regs.hpp"
 
 static const std::string X300_FW_FILE_NAME  = "usrp_x300_fw.bin";
 
 static const double X300_DEFAULT_TICK_RATE      = 200e6;        //Hz
 static const double X300_BUS_CLOCK_RATE         = 166.666667e6; //Hz
 
-static const size_t X300_TX_HW_BUFF_SIZE        = 0x90000;      //576KiB
+static const size_t X300_TX_HW_BUFF_SIZE        = 520*1024;      //512K SRAM buffer + 8K 2Clk FIFO
 static const size_t X300_TX_FC_RESPONSE_FREQ    = 8;            //per flow-control window
 
 static const size_t X300_RX_SW_BUFF_SIZE_ETH        = 0x2000000;//32MiB    For an ~8k frame size any size >32MiB is just wasted buffer space
@@ -70,13 +71,13 @@ static const size_t X300_PCIE_RX_DATA_FRAME_SIZE    = 8184;     //bytes
 static const size_t X300_PCIE_TX_DATA_FRAME_SIZE    = 8192;     //bytes
 static const size_t X300_PCIE_DATA_NUM_FRAMES       = 2048;
 static const size_t X300_PCIE_MSG_FRAME_SIZE        = 256;      //bytes
-static const size_t X300_PCIE_MSG_NUM_FRAMES        = 32;
+static const size_t X300_PCIE_MSG_NUM_FRAMES        = 64;
 
 static const size_t X300_10GE_DATA_FRAME_MAX_SIZE   = 8000;     //bytes
 static const size_t X300_1GE_DATA_FRAME_MAX_SIZE    = 1472;     //bytes
 static const size_t X300_ETH_MSG_FRAME_SIZE         = uhd::transport::udp_simple::mtu;  //bytes
 
-static const size_t X300_ETH_MSG_NUM_FRAMES         = 32;
+static const size_t X300_ETH_MSG_NUM_FRAMES         = 64;
 static const size_t X300_ETH_DATA_NUM_FRAMES        = 32;
 static const double X300_DEFAULT_SYSREF_RATE        = 10e6;
 
@@ -131,13 +132,16 @@ struct x300_dboard_iface_config_t
     x300_clock_which_t which_rx_clk;
     x300_clock_which_t which_tx_clk;
     boost::uint8_t dboard_slot;
+    uhd::timed_wb_iface::sptr cmd_time_ctrl;
 };
 
 uhd::usrp::dboard_iface::sptr x300_make_dboard_iface(const x300_dboard_iface_config_t &);
 uhd::uart_iface::sptr x300_make_uart_iface(uhd::wb_iface::sptr iface);
 
 uhd::wb_iface::sptr x300_make_ctrl_iface_enet(uhd::transport::udp_simple::sptr udp);
-uhd::wb_iface::sptr x300_make_ctrl_iface_pcie(uhd::niusrprio::niriok_proxy& drv_proxy);
+uhd::wb_iface::sptr x300_make_ctrl_iface_pcie(uhd::niusrprio::niriok_proxy::sptr drv_proxy);
+
+uhd::device_addrs_t x300_find(const uhd::device_addr_t &hint_);
 
 class x300_impl : public uhd::device
 {
@@ -171,6 +175,7 @@ private:
     //perifs in the radio core
     struct radio_perifs_t
     {
+        //Interfaces
         radio_ctrl_core_3000::sptr ctrl;
         spi_core_3000::sptr spi;
         x300_adc_ctrl::sptr adc;
@@ -183,6 +188,8 @@ private:
         gpio_core_200_32wo::sptr leds;
         rx_frontend_core_200::sptr rx_fe;
         tx_frontend_core_200::sptr tx_fe;
+        //Registers
+        uhd::usrp::x300::radio_regmap_t::sptr regmap;
     };
 
     //overflow recovery impl
@@ -194,6 +201,7 @@ private:
         uhd::dict<size_t, boost::weak_ptr<uhd::rx_streamer> > rx_streamers;
         uhd::dict<size_t, boost::weak_ptr<uhd::tx_streamer> > tx_streamers;
 
+        bool initialization_done;
         uhd::task::sptr claimer_task;
         std::string addr;
         std::string xport_path;
@@ -209,7 +217,8 @@ private:
         i2c_core_100_wb32::sptr zpu_i2c;
 
         //perifs in each radio
-        radio_perifs_t radio_perifs[2]; //!< This is hardcoded s.t. radio_perifs[0] points to slot A and [1] to B
+        static const size_t NUM_RADIOS = 2;
+        radio_perifs_t radio_perifs[NUM_RADIOS]; //!< This is hardcoded s.t. radio_perifs[0] points to slot A and [1] to B
         uhd::usrp::dboard_eeprom_t db_eeproms[8];
         //! Return the index of a radio component, given a slot name. This means DSPs, radio_perifs
         size_t get_radio_index(const std::string &slot_name) {
@@ -222,17 +231,15 @@ private:
         uhd::gps_ctrl::sptr gps;
         gpio_core_200::sptr fp_gpio;
 
-        //clock control register bits
-        int clock_control_regs_clock_source;
-        int clock_control_regs_pps_select;
-        int clock_control_regs_pps_out_enb;
-        int clock_control_regs_tcxo_enb;
-        int clock_control_regs_gpsdo_pwr;
+        uhd::usrp::x300::fw_regmap_t::sptr fw_regmap;
 
         //which FPGA image is loaded
         std::string loaded_fpga_image;
 
         size_t hw_rev;
+        std::string current_refclk_src;
+
+        uhd::soft_regmap_db_t::sptr regmap_db;
     };
     std::vector<mboard_members_t> _mb;
 
@@ -243,6 +250,8 @@ private:
 
     void register_loopback_self_test(uhd::wb_iface::sptr iface);
 
+    void radio_loopback(uhd::wb_iface::sptr iface, const bool on);
+
      /*! \brief Initialize the radio component on a given slot.
       *
       * Call this function once per slot (A and B) and motherboard to initialize all the radio components.
@@ -256,7 +265,7 @@ private:
       * \param mb_i Motherboard index
       * \param slot_name Slot name (A or B).
       */
-    void setup_radio(const size_t, const std::string &slot_name);
+    void setup_radio(const size_t, const std::string &slot_name, const uhd::device_addr_t &dev_addr);
 
     size_t _sid_framer;
     struct sid_config_t
@@ -322,6 +331,7 @@ private:
 
     void set_rx_fe_corrections(const uhd::fs_path &mb_path, const std::string &fe_name, const double lo_freq);
     void set_tx_fe_corrections(const uhd::fs_path &mb_path, const std::string &fe_name, const double lo_freq);
+    bool _ignore_cal_file;
 
 
     /*! Update the IQ MUX settings for the radio peripheral according to given subdev spec.
@@ -340,25 +350,36 @@ private:
     void update_tx_samp_rate(mboard_members_t&, const size_t, const double);
 
     void update_clock_control(mboard_members_t&);
+    void initialize_clock_control(mboard_members_t &mb);
     void set_time_source_out(mboard_members_t&, const bool);
     void update_clock_source(mboard_members_t&, const std::string &);
     void update_time_source(mboard_members_t&, const std::string &);
-    void reset_clocks(mboard_members_t&);
-    void reset_radios(mboard_members_t&);
 
-    uhd::sensor_value_t get_ref_locked(uhd::wb_iface::sptr);
-    void wait_for_ref_locked(uhd::wb_iface::sptr, double timeout = 0.0);
-    bool is_pps_present(uhd::wb_iface::sptr);
+    uhd::sensor_value_t get_ref_locked(mboard_members_t& mb);
+    bool wait_for_clk_locked(mboard_members_t& mb, boost::uint32_t which, double timeout);
+    bool is_pps_present(mboard_members_t& mb);
 
     void set_db_eeprom(uhd::i2c_iface::sptr i2c, const size_t, const uhd::usrp::dboard_eeprom_t &);
     void set_mb_eeprom(uhd::i2c_iface::sptr i2c, const uhd::usrp::mboard_eeprom_t &);
 
     void check_fw_compat(const uhd::fs_path &mb_path, uhd::wb_iface::sptr iface);
-    void check_fpga_compat(const uhd::fs_path &mb_path, uhd::wb_iface::sptr iface);
+    void check_fpga_compat(const uhd::fs_path &mb_path, const mboard_members_t &members);
 
     void update_atr_leds(gpio_core_200_32wo::sptr, const std::string &ant);
-    boost::uint32_t get_fp_gpio(gpio_core_200::sptr, const std::string &);
-    void set_fp_gpio(gpio_core_200::sptr, const std::string &, const boost::uint32_t);
+    boost::uint32_t get_fp_gpio(gpio_core_200::sptr);
+    void set_fp_gpio(gpio_core_200::sptr, const gpio_attr_t, const boost::uint32_t);
+
+    void self_cal_adc_capture_delay(mboard_members_t& mb, const size_t radio_i, bool print_status = false);
+    double self_cal_adc_xfer_delay(mboard_members_t& mb, bool apply_delay = false);
+    void self_test_adcs(mboard_members_t& mb, boost::uint32_t ramp_time_ms = 100);
+
+    void extended_adc_test(mboard_members_t& mb, double duration_s);
+
+    //**PRECONDITION**
+    //This function assumes that all the VITA times in "radios" are synchronized
+    //to a common reference. Currently, this function is called in get_tx_stream
+    //which also has the same precondition.
+    static void synchronize_dacs(const std::vector<radio_perifs_t*>& mboards);
 };
 
 #endif /* INCLUDED_X300_IMPL_HPP */
